package java.io;

import sun.misc.JavaIOAccess;
import sun.misc.SharedSecrets;
import sun.nio.cs.StreamDecoder;
import sun.nio.cs.StreamEncoder;
import java.util.Arrays;
import java.nio.charset.Charset;
import java.util.Formatter;

public final class Console implements Flushable
{
    private Object readLock;
    private Object writeLock;
    private Reader reader;
    private Writer out;
    private PrintWriter pw;
    private Formatter formatter;
    private Charset cs;
    private char[] rcb;
    private static boolean echoOff;
    private static Console cons;
    
    public PrintWriter writer() {
        return this.pw;
    }
    
    public Reader reader() {
        return this.reader;
    }
    
    public Console format(String format, Object... args) {
        this.formatter.format(format, args).flush();
        return this;
    }
    
    public Console printf(String s, Object... array) {
        return this.format(s, array);
    }
    
    public String readLine(String s, Object... array) {
        String s2 = null;
        synchronized (this.writeLock) {
            synchronized (this.readLock) {
                if (s.length() != 0) {
                    this.pw.format(s, array);
                }
                try {
                    char[] readline = this.readline(false);
                    if (readline != null) {
                        s2 = new String(readline);
                    }
                }
                catch (IOException ex) {
                    throw new IOError(ex);
                }
            }
        }
        return s2;
    }
    
    public String readLine() {
        return this.readLine("", new Object[0]);
    }
    
    public char[] readPassword(String s, Object... array) {
        char[] readline = null;
        synchronized (this.writeLock) {
            synchronized (this.readLock) {
                try {
                    Console.echoOff = echo(false);
                }
                catch (IOException ex) {
                    throw new IOError(ex);
                }
                IOException ex = null;
                try {
                    if (s.length() != 0) {
                        this.pw.format(s, array);
                    }
                    readline = this.readline(true);
                    try {
                        Console.echoOff = echo(true);
                    }
                    catch (IOException exception) {
                        if (ex == null) {
                            ex = new IOError(exception);
                        }
                        else {
                            ex.addSuppressed(exception);
                        }
                    }
                    if (ex != null) {
                        throw ex;
                    }
                }
                catch (IOException ex2) {
                    ex = new IOError(ex2);
                    try {
                        Console.echoOff = echo(true);
                    }
                    catch (IOException exception2) {
                        if (ex == null) {
                            ex = new IOError(exception2);
                        }
                        else {
                            ex.addSuppressed(exception2);
                        }
                    }
                    if (ex != null) {
                        throw ex;
                    }
                }
                finally {
                    try {
                        Console.echoOff = echo(true);
                    }
                    catch (IOException exception3) {
                        if (ex == null) {
                            ex = new IOError(exception3);
                        }
                        else {
                            ex.addSuppressed(exception3);
                        }
                    }
                    if (ex != null) {
                        throw ex;
                    }
                }
                this.pw.println();
            }
        }
        return readline;
    }
    
    public char[] readPassword() {
        return this.readPassword("", new Object[0]);
    }
    
    @Override
    public void flush() {
        this.pw.flush();
    }
    
    private static native String encoding();
    
    private static native boolean echo(boolean p0) throws IOException;
    
    private char[] readline(boolean b) throws IOException {
        int read = this.reader.read(this.rcb, 0, this.rcb.length);
        if (read < 0) {
            return null;
        }
        if (this.rcb[read - 1] == '\r') {
            --read;
        }
        else if (this.rcb[read - 1] == '\n' && --read > 0 && this.rcb[read - 1] == '\r') {
            --read;
        }
        char[] array = new char[read];
        if (read > 0) {
            System.arraycopy(this.rcb, 0, array, 0, read);
            if (b) {
                Arrays.fill(this.rcb, 0, read, ' ');
            }
        }
        return array;
    }
    
    private char[] grow() {
        assert !(!Thread.holdsLock(this.readLock));
        char[] rcb = new char[this.rcb.length * 2];
        System.arraycopy(this.rcb, 0, rcb, 0, this.rcb.length);
        return this.rcb = rcb;
    }
    
    private static native boolean istty();
    
    private Console() {
        this.readLock = new Object();
        this.writeLock = new Object();
        String encoding = encoding();
        if (encoding != null) {
            try {
                this.cs = Charset.forName(encoding);
            }
            catch (Exception ex) {}
        }
        if (this.cs == null) {
            this.cs = Charset.defaultCharset();
        }
        this.out = StreamEncoder.forOutputStreamWriter(new FileOutputStream(FileDescriptor.out), this.writeLock, this.cs);
        this.pw = new PrintWriter(this.out, true) {
            @Override
            public void close() {
            }
        };
        this.formatter = new Formatter(this.out);
        this.reader = new Console.LineReader(StreamDecoder.forInputStreamReader(new FileInputStream(FileDescriptor.in), this.readLock, this.cs));
        this.rcb = new char[1024];
    }
    
    static char[] access$200(Console console) {
        return console.grow();
    }
    
    static boolean access$400(boolean b) throws IOException {
        return echo(b);
    }
    
    static boolean access$500() {
        return istty();
    }
    
    Console(Console.1 runnable) {
        this();
    }
    
    static {
        try {
            SharedSecrets.getJavaLangAccess().registerShutdownHook(0, false, new Runnable() {
                @Override
                public void run() {
                    try {
                        if (Console.echoOff) {
                            Console.access$400(true);
                        }
                    }
                    catch (IOException ex) {}
                }
            });
        }
        catch (IllegalStateException ex) {}
        SharedSecrets.setJavaIOAccess(new JavaIOAccess() {
            public Console console() {
                if (Console.access$500()) {
                    if (Console.cons == null) {
                        Console.cons = new Console();
                    }
                    return Console.cons;
                }
                else {
                    return null;
                }
            }
            
            public Charset charset() {
                return Console.cons.cs;
            }
        });
    }
    
    class LineReader extends Reader
    {
        private Reader in;
        private char[] cb;
        private int nChars;
        private int nextChar;
        boolean leftoverLF;
        
        LineReader(Reader in) {
            this.in = in;
            this.cb = new char[1024];
            int n = 0;
            this.nChars = n;
            this.nextChar = n;
            this.leftoverLF = false;
        }
        
        @Override
        public void close() {
        }
        
        @Override
        public boolean ready() throws IOException {
            return this.in.ready();
        }
        
        @Override
        public int read(char[] array, int n, int n2) throws IOException {
            int length = n + n2;
            if (n < 0 || n > array.length || n2 < 0 || length < 0 || length > array.length) {
                throw new IndexOutOfBoundsException();
            }
            synchronized (Console.this.readLock) {
                boolean n3 = false;
                do {
                    if (this.nextChar >= this.nChars) {
                        int i;
                        do {
                            i = this.in.read(this.cb, 0, this.cb.length);
                        } while (i == 0);
                        if (i > 0) {
                            this.nChars = i;
                            this.nextChar = 0;
                            if (i < this.cb.length && this.cb[i - 1] != '\n' && this.cb[i - 1] != '\r') {
                                n3 = true;
                            }
                        }
                        else {
                            if (n - n == 0) {
                                return -1;
                            }
                            return n - n;
                        }
                    }
                    if (this.leftoverLF && array == Console.this.rcb && this.cb[this.nextChar] == '\n') {
                        ++this.nextChar;
                    }
                    this.leftoverLF = false;
                    while (this.nextChar < this.nChars) {
                        char[] array2 = array;
                        int n6 = n++;
                        char c = this.cb[this.nextChar];
                        array2[n6] = c;
                        int n7 = c;
                        this.cb[this.nextChar++] = '\0';
                        if (n7 == '\n') {
                            return n - n;
                        }
                        if (n7 == '\r') {
                            if (n == length) {
                                if (array == Console.this.rcb) {
                                    array = Console.access$200(Console.this);
                                    int length2 = array.length;
                                }
                                else {
                                    this.leftoverLF = true;
                                    return n - n;
                                }
                            }
                            if (this.nextChar == this.nChars && this.in.ready()) {
                                this.nChars = this.in.read(this.cb, 0, this.cb.length);
                                this.nextChar = 0;
                            }
                            if (this.nextChar < this.nChars && this.cb[this.nextChar] == '\n') {
                                array[n++] = '\n';
                                ++this.nextChar;
                            }
                            return n - n;
                        }
                        else {
                            if (n != length) {
                                continue;
                            }
                            if (array == Console.this.rcb) {
                                array = Console.access$200(Console.this);
                                length = array.length;
                            }
                            else {
                                return n - n;
                            }
                        }
                    }
                } while (n3 == 0);
                return n - n;
            }
        }
    }
}
